@@ -24,7 +24,9 @@ finding conjunctions using the south B-trace data
 # function and class imports
 from .conjunctions import (search,
                            search_async,
-                           describe)
+                           describe,
+                           get_request_url)
+from .swarmaurora import __all__ as swarmaurora_all
 from .classes.conjunction import Conjunction
 from .classes.search import Search
 
@@ -41,6 +43,8 @@ __all__ = [
     "search",
     "search_async",
     "describe",
+    "get_request_url",
     "Conjunction",
     "Search",
+    "swarmaurora_all",
 ]
