@@ -5,7 +5,6 @@ Class definition for a statistics about a data source
 import datetime
 from pydantic import BaseModel
 from typing import Dict, Optional
-from .data_source import DataSource
 
 # pdoc init
 __pdoc__: Dict = {}
@@ -16,7 +15,6 @@ class DataSourceStatistics(BaseModel):
     Data source statistics object
 
     Attributes:
-        data_source: the data source the statistics are associated with
         earliest_ephemeris_loaded: timestamp of the earliest ephemeris record
         latest_ephemeris_loaded: timestamp of the latest ephemeris record
         ephemeris_count: total number of ephemeris records for this data source
@@ -24,7 +22,6 @@ class DataSourceStatistics(BaseModel):
         latest_data_product_loaded: timestamp of the latest data product record
         data_product_count: total number of ephemeris records for this data source
     """
-    data_source: DataSource
     earliest_ephemeris_loaded: Optional[datetime.datetime] = None
     latest_ephemeris_loaded: Optional[datetime.datetime] = None
     ephemeris_count: int
