@@ -3,7 +3,7 @@ Functions for interacting with data sources
 """
 
 import warnings
-from typing import List, Dict, Optional
+from typing import List, Dict, Optional, Any
 from .classes.data_source import DataSource
 from .classes.data_source_stats import DataSourceStatistics
 from ..sources import FORMAT_FULL_RECORD
@@ -24,7 +24,8 @@ def list(program: Optional[str] = None,
          source_type: Optional[str] = None,
          owner: Optional[str] = None,
          format: Optional[str] = FORMAT_FULL_RECORD,
-         order: Optional[str] = "identifier") -> List[DataSource]:
+         order: Optional[str] = "identifier",
+         include_stats: Optional[bool] = False) -> List[DataSource]:
     """
     Retrieve all data source records (using params to filter as desired)
 
@@ -41,6 +42,9 @@ def list(program: Optional[str] = None,
             using the pyaurorax.FORMAT_* variables.
         order: the value to order results by (identifier, program, platform,
             instrument_type, display_name, owner), defaults to "identifier"
+        include_stats: include additional stats information about the data source (note:
+            slower response time since an additional request must be done for each
+            data source), defaults to False
 
     Returns:
         any data sources matching the requested parameters
@@ -66,18 +70,28 @@ def list(program: Optional[str] = None,
     # order results
     res.data = sorted(res.data, key=lambda x: x[order])
 
-    # return
+    # cast
     if len(res.data):
-        return [DataSource(**ds, format=format) for ds in res.data]
+        data_sources = [DataSource(**ds, format=format) for ds in res.data]
     else:
         return []
 
+    # get stats if requested
+    if (include_stats is True):
+        for i in range(0, len(data_sources)):
+            data_source_with_stats = get_stats(data_sources[i].identifier)
+            data_sources[i].stats = data_source_with_stats.stats
+
+    # return
+    return data_sources
+
 
 def search(programs: Optional[List[str]] = [],
            platforms: Optional[List[str]] = [],
            instrument_types: Optional[List[str]] = [],
            format: Optional[str] = FORMAT_FULL_RECORD,
-           order: Optional[str] = "identifier") -> List[DataSource]:
+           order: Optional[str] = "identifier",
+           include_stats: Optional[bool] = False) -> List[DataSource]:
     """
     Search for data source records (using params to filter as desired)
 
@@ -94,6 +108,9 @@ def search(programs: Optional[List[str]] = [],
             using the pyaurorax.FORMAT_* variables.
         order: the value to order results by (identifier, program, platform,
             instrument_type, display_name), defaults to "identifier"
+        include_stats: include additional stats information about the data source (note:
+            slower response time since an additional request must be done for each
+            data source), defaults to False
 
     Returns:
         any data sources matching the requested parameters
@@ -120,17 +137,27 @@ def search(programs: Optional[List[str]] = [],
     # order results
     res.data = sorted(res.data, key=lambda x: x[order])
 
-    # return
+    # cast
     if len(res.data):
-        return [DataSource(**ds, format=format) for ds in res.data]
+        data_sources = [DataSource(**ds, format=format) for ds in res.data]
     else:
         return []
 
+    # get stats if requested
+    if (include_stats is True):
+        for i in range(0, len(data_sources)):
+            data_source_with_stats = get_stats(data_sources[i].identifier)
+            data_sources[i].stats = data_source_with_stats.stats
+
+    # return
+    return data_sources
+
 
 def get(program: str,
         platform: str,
         instrument_type: str,
-        format: Optional[str] = FORMAT_FULL_RECORD) -> DataSource:
+        format: Optional[str] = FORMAT_FULL_RECORD,
+        include_stats: Optional[bool] = False) -> DataSource:
     """
     Retrieve a specific data source record
 
@@ -141,6 +168,9 @@ def get(program: str,
         format: the format of the data sources returned, defaults to "full_record".
             Other options are in the pyaurorax.sources module, or at the top level
             using the pyaurorax.FORMAT_* variables.
+        include_stats: include additional stats information about the data source (note:
+            slower response time since an additional request must be done for each
+            data source), defaults to False
 
     Returns:
         the data source matching the requested parameters
@@ -154,7 +184,8 @@ def get(program: str,
     data_sources = list(program=program,
                         platform=platform,
                         instrument_type=instrument_type,
-                        format=format)
+                        format=format,
+                        include_stats=include_stats)
 
     # set results to the first thing found
     if (len(data_sources) == 1):
@@ -173,7 +204,8 @@ def get_using_filters(program: Optional[str] = None,
                       source_type: Optional[str] = None,
                       owner: Optional[str] = None,
                       format: Optional[str] = FORMAT_FULL_RECORD,
-                      order: Optional[str] = "identifier") -> List[DataSource]:
+                      order: Optional[str] = "identifier",
+                      include_stats: Optional[bool] = False) -> List[DataSource]:
     """
     Retrieve all data source records matching a filter
 
@@ -190,6 +222,9 @@ def get_using_filters(program: Optional[str] = None,
             using the pyaurorax.FORMAT_* variables.
         order: the value to order results by (identifier, program, platform,
             instrument_type, display_name, owner), defaults to "identifier"
+        include_stats: include additional stats information about the data source (note:
+            slower response time since an additional request must be done for each
+            data source), defaults to False
 
     Returns:
         any data sources matching the requested parameters
@@ -205,14 +240,16 @@ def get_using_filters(program: Optional[str] = None,
                         source_type=source_type,
                         owner=owner,
                         format=format,
-                        order=order)
+                        order=order,
+                        include_stats=include_stats)
 
     # return
     return data_sources
 
 
 def get_using_identifier(identifier: int,
-                         format: Optional[str] = FORMAT_FULL_RECORD) -> DataSource:
+                         format: Optional[str] = FORMAT_FULL_RECORD,
+                         include_stats: Optional[bool] = False) -> DataSource:
     """
     Retrieve data source record matching an identifier
 
@@ -221,6 +258,9 @@ def get_using_identifier(identifier: int,
         format: the format of the data sources returned, defaults to "full_record".
             Other options are in the pyaurorax.sources module, or at the top level
             using the pyaurorax.FORMAT_* variables.
+        include_stats: include additional stats information about the data source (note:
+            slower response time since an additional request must be done for each
+            data source), defaults to False
 
     Returns:
         the data source matching the identifier
@@ -237,13 +277,20 @@ def get_using_identifier(identifier: int,
     req = AuroraXRequest(method="get", url=url, params=params)
     res = req.execute()
 
+    # cast
+    data_source = DataSource(**res.data, format=format)
+
+    # get stats if requested
+    if (include_stats is True):
+        data_source = get_stats(data_source.identifier)
+
     # return
-    return DataSource(**res.data, format=format)
+    return data_source
 
 
 def get_stats(identifier: int,
               format: Optional[str] = FORMAT_FULL_RECORD,
-              slow: Optional[bool] = False) -> DataSourceStatistics:
+              slow: Optional[bool] = False) -> DataSource:
     """
     Retrieve statistics for a data source
 
@@ -255,7 +302,7 @@ def get_stats(identifier: int,
         slow: retrieve the stats using a slower, but more accurate method, defaults to False
 
     Returns:
-        information about the data source
+        the data source including additional stats information about it
 
     Raises:
         pyaurorax.exceptions.AuroraXMaxRetriesException: max retry error
@@ -272,11 +319,12 @@ def get_stats(identifier: int,
     res = req.execute()
 
     # cast data source record
+    stats = DataSourceStatistics(**res.data)
     ds = DataSource(**res.data["data_source"], format=format)
-    res.data["data_source"] = ds
+    ds.stats = stats
 
     # return
-    return DataSourceStatistics(**res.data)
+    return ds
 
 
 def add(data_source: DataSource) -> DataSource:
@@ -305,7 +353,7 @@ def add(data_source: DataSource) -> DataSource:
         "ephemeris_metadata_schema": data_source.ephemeris_metadata_schema,
         "data_product_metadata_schema": data_source.data_product_metadata_schema,
         "metadata": data_source.metadata
-    }
+    }  # type: Dict[str, Any]
     if (data_source.identifier is not None):
         request_data["identifier"] = data_source.identifier
 
@@ -399,6 +447,7 @@ def update(data_source: DataSource) -> DataSource:
 
     # make request to update the data source passed in
     req_data = data_source.__dict__
+    del req_data["stats"]
     del req_data["format"]
     req = AuroraXRequest(method="put", url=url, body=req_data)
     req.execute()
@@ -470,6 +519,7 @@ def update_partial(identifier: int,
     # set URL and request data
     url = f"{urls.data_sources_url}/{ds.identifier}"
     req_data = ds.__dict__
+    del req_data["stats"]
     del req_data["format"]
 
     # make request
